import { ReturnModelType } from '@typegoose/typegoose';
import { AnyParamConstructor } from '@typegoose/typegoose/lib/types';
import { Application, Context } from '@midwayjs/koa';
import { App, Inject } from '@midwayjs/decorator';
import { FilterDTO } from './BaseTypes';
import _ = require('lodash');
import { FrameworkErrorEnum, ServiceError } from '../error/service.error';
/**
 * Service基类
 */
export abstract class BaseService<T extends AnyParamConstructor<any>> {
  @App()
  app: Application;
  @Inject()
  ctx: Context;

  // 要求继承基类的service,必须给上默认的model
  abstract model: ReturnModelType<T>;

  /**
   * 常规列表查询
   * @param {FilterDTO} filter 查询条件
   * @param {object} pageOptions 分页条件
   * @param {Boolean} lean 是否使用JavaScript形式数据;false为mongoose的模型实例数据
   * @returns {Promise<object>} 返回列表
   */
  async query(filter: FilterDTO = {}, pageOptions: object = {}, lean = true): Promise<object> {
    const dup = _.cloneDeep(filter);
    const data = await this.model.find(dup, {}, { ...pageOptions }).lean(lean);
    return data;
  }

  async count(filter: FilterDTO = {}) {
    const dup = _.cloneDeep(filter);
    const total = await this.model.count(dup);
    return total;
  }

  async fetch(id: string, lean = true) {
    const data = await this.model.findById(id).lean(lean);
    return data;
  }

  async updateOne(id: string, body: object) {
    if (!id) throw new ServiceError('缺少查询信息', FrameworkErrorEnum.NEED_PARAMS);
    const num = await this.model.count({ _id: id });
    if (num <= 0) throw new ServiceError('未找到要修改的数据', FrameworkErrorEnum.NOT_FOUND_DATA);
    const data = await this.model.updateOne({ _id: id }, body);
    return data;
  }

  async delete(filter: FilterDTO) {}

  async create(body: object) {
    const data = await this.model.create(body);
    return data;
  }
}
